@@ -24,6 +24,7 @@
 #include "processcontrol.h"
 #include "serverinterface.h"
 #include "../../libs/xdgbasedirs_p.h"
+#include "akdebug.h"
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
@@ -50,6 +51,8 @@ AgentManager::AgentManager( QObject *parent )
     qFatal( "akonadiserver already running!" );
 
   mStorageController = new Akonadi::ProcessControl;
+  connect( mStorageController, SIGNAL(unableToStart()),
+           QCoreApplication::instance(), SLOT(quit()) );
   mStorageController->start( "akonadiserver", QStringList(), Akonadi::ProcessControl::RestartOnCrash );
 }
 
@@ -62,7 +65,7 @@ void AgentManager::continueStartup()
   first = false;
 
   readPluginInfos();
-  foreach ( const AgentInfo &info, mAgents )
+  foreach ( const AgentType &info, mAgents )
     emit agentTypeAdded( info.identifier );
 
   QStringList pathList = pluginInfoPathList();
@@ -76,7 +79,7 @@ void AgentManager::continueStartup()
   }
 
   load();
-  foreach ( const AgentInfo &info, mAgents )
+  foreach ( const AgentType &info, mAgents )
     ensureAutoStart( info );
 }
 
@@ -87,11 +90,8 @@ AgentManager::~AgentManager()
 
 void AgentManager::cleanup()
 {
-  foreach ( const AgentInstanceInfo &info, mAgentInstances ) {
-    info.controller->setCrashPolicy( ProcessControl::StopOnCrash );
-    if ( info.agentControlInterface && info.agentControlInterface->isValid() )
-      info.agentControlInterface->quit();
-  }
+  foreach ( const AgentInstance::Ptr &inst, mAgentInstances )
+    inst->quit();
 
   mAgentInstances.clear();
 
@@ -128,7 +128,7 @@ QString AgentManager::agentIcon( const QString &identifier ) const
 {
   if ( !checkAgentExists( identifier ) )
     return QString();
-  const AgentInfo info = mAgents.value( identifier );
+  const AgentType info = mAgents.value( identifier );
   if ( !info.icon.isEmpty() )
     return info.icon;
   return "application-x-executable";
@@ -152,29 +152,28 @@ QString AgentManager::createAgentInstance( const QString &identifier )
 {
   if ( !checkAgentExists( identifier ) )
     return QString();
-  AgentInfo agentInfo = mAgents.value( identifier );
+  AgentType agentInfo = mAgents.value( identifier );
   mAgents[ identifier ].instanceCounter++;
 
 
-  AgentInstanceInfo instance;
-  if ( agentInfo.capabilities.contains( AgentInfo::CapabilityUnique ) )
-    instance.identifier = identifier;
+  AgentInstance::Ptr instance( new AgentInstance( this ) );
+  if ( agentInfo.capabilities.contains( AgentType::CapabilityUnique ) )
+    instance->setIdentifier( identifier );
   else
-    instance.identifier = QString::fromLatin1( "%1_%2" ).arg( identifier, QString::number( agentInfo.instanceCounter ) );
-  instance.agentType = identifier;
+    instance->setIdentifier( QString::fromLatin1( "%1_%2" ).arg( identifier, QString::number( agentInfo.instanceCounter ) ) );
 
-  if ( mAgentInstances.contains( instance.identifier ) ) {
+  if ( mAgentInstances.contains( instance->identifier() ) ) {
     mTracer->warning( QLatin1String("AgentManager::createAgentInstance"),
                       QString::fromLatin1( "Cannot create another instance of agent '%1'." ).arg( identifier ) );
     return QString();
   }
 
-  if ( !instance.start( agentInfo, this ) )
+  if ( !instance->start( agentInfo ) )
     return QString();
-  mAgentInstances.insert( instance.identifier, instance );
+  mAgentInstances.insert( instance->identifier(), instance );
 
   save();
-  return instance.identifier;
+  return instance->identifier();
 }
 
 void AgentManager::removeAgentInstance( const QString &identifier )
@@ -185,19 +184,14 @@ void AgentManager::removeAgentInstance( const QString &identifier )
     return;
   }
 
-  AgentInstanceInfo instance = mAgentInstances.value( identifier );
-  if ( instance.agentControlInterface ) {
-    instance.agentControlInterface->cleanup();
+  AgentInstance::Ptr instance = mAgentInstances.value( identifier );
+  if ( instance->hasAgentInterface() ) {
+    instance->cleanup();
   } else {
     mTracer->error( QLatin1String( "AgentManager::removeAgentInstance" ),
                     QString( "Agent instance '%1' has no interface!" ).arg( identifier ) );
   }
 
-  delete instance.resourceInterface;
-  delete instance.agentControlInterface;
-  delete instance.agentStatusInterface;
-  delete instance.controller;
-
   mAgentInstances.remove( identifier );
 
   save();
@@ -213,7 +207,7 @@ QString AgentManager::agentInstanceType( const QString &identifier )
     return QString();
   }
 
-  return mAgentInstances.value( identifier ).agentType;
+  return mAgentInstances.value( identifier )->agentType();
 }
 
 QStringList AgentManager::agentInstances() const
@@ -223,23 +217,23 @@ QStringList AgentManager::agentInstances() const
 
 int AgentManager::agentInstanceStatus( const QString &identifier ) const
 {
-  if ( !checkAgentInterfaces( identifier, QLatin1String( "agentInstanceStatus" ) ) )
+  if ( !checkInstance( identifier ) )
     return 2;
-  return mAgentInstances.value( identifier ).agentStatusInterface->status();
+  return mAgentInstances.value( identifier )->status();
 }
 
 QString AgentManager::agentInstanceStatusMessage( const QString &identifier ) const
 {
-  if ( !checkAgentInterfaces( identifier, QLatin1String( "agentInstanceStatusMessage" ) ) )
+  if ( !checkInstance( identifier ) )
     return QString();
-  return mAgentInstances.value( identifier ).agentStatusInterface->statusMessage();
+  return mAgentInstances.value( identifier )->statusMessage();
 }
 
 uint AgentManager::agentInstanceProgress( const QString &identifier ) const
 {
-  if ( !checkAgentInterfaces( identifier, QLatin1String( "agentInstanceProgress" ) ) )
+  if ( !checkInstance( identifier ) )
     return 0;
-  return mAgentInstances.value( identifier ).agentStatusInterface->progress();
+  return mAgentInstances.value( identifier )->progress();
 }
 
 QString AgentManager::agentInstanceProgressMessage( const QString &identifier ) const
@@ -251,21 +245,21 @@ void AgentManager::agentInstanceConfigure( const QString &identifier, qlonglong
 {
   if ( !checkAgentInterfaces( identifier, "agentInstanceConfigure" ) )
     return;
-  mAgentInstances.value( identifier ).agentControlInterface->configure( windowId );
+  mAgentInstances.value( identifier )->controlInterface()->configure( windowId );
 }
 
 bool AgentManager::agentInstanceOnline(const QString & identifier)
 {
-  if ( !checkAgentInterfaces( identifier, QLatin1String( "agentInstanceOnline" ) ) )
+  if ( !checkInstance( identifier ) )
     return false;
-  return mAgentInstances.value( identifier ).agentStatusInterface->isOnline();
+  return mAgentInstances.value( identifier )->isOnline();
 }
 
 void AgentManager::setAgentInstanceOnline(const QString & identifier, bool state )
 {
   if ( !checkAgentInterfaces( identifier, QLatin1String( "setAgentInstanceOnline" ) ) )
     return;
-  mAgentInstances.value( identifier ).agentStatusInterface->setOnline( state );
+  mAgentInstances.value( identifier )->statusInterface()->setOnline( state );
 }
 
 // resource specific methods //
@@ -273,53 +267,53 @@ void AgentManager::setAgentInstanceName( const QString &identifier, const QStrin
 {
   if ( !checkResourceInterface( identifier, QLatin1String( "setAgentInstanceName" ) ) )
     return;
-  mAgentInstances.value( identifier ).resourceInterface->setName( name );
+  mAgentInstances.value( identifier )->resourceInterface()->setName( name );
 }
 
 QString AgentManager::agentInstanceName( const QString &identifier ) const
 {
   if ( !checkInstance( identifier ) )
     return QString();
-  const AgentInstanceInfo inst = mAgentInstances.value( identifier );
-  if ( inst.isResource() )
-    return inst.resourceInterface->name();
-  if ( !checkAgentExists( inst.agentType ) )
+  const AgentInstance::Ptr inst = mAgentInstances.value( identifier );
+  if ( !inst->resourceName().isEmpty() )
+    return inst->resourceName();
+  if ( !checkAgentExists( inst->agentType() ) )
     return QString();
-  return mAgents.value( inst.agentType ).name;
+  return mAgents.value( inst->agentType() ).name;
 }
 
 void AgentManager::agentInstanceSynchronize( const QString &identifier )
 {
   if ( !checkResourceInterface( identifier, QLatin1String( "agentInstanceSynchronize" ) ) )
     return;
-  mAgentInstances.value( identifier ).resourceInterface->synchronize();
+  mAgentInstances.value( identifier )->resourceInterface()->synchronize();
 }
 
 void AgentManager::agentInstanceSynchronizeCollectionTree(const QString & identifier)
 {
   if ( !checkResourceInterface( identifier, QLatin1String( "agentInstanceSynchronizeCollectionTree" ) ) )
     return;
-  mAgentInstances.value( identifier ).resourceInterface->synchronizeCollectionTree();
+  mAgentInstances.value( identifier )->resourceInterface()->synchronizeCollectionTree();
 }
 
 void AgentManager::agentInstanceSynchronizeCollection(const QString & identifier, qint64 collection)
 {
   if ( !checkResourceInterface( identifier, QLatin1String( "agentInstanceSynchronizeCollection" ) ) )
     return;
-  mAgentInstances.value( identifier ).resourceInterface->synchronizeCollection( collection );
+  mAgentInstances.value( identifier )->resourceInterface()->synchronizeCollection( collection );
 }
 
 void AgentManager::updatePluginInfos()
 {
-  QHash<QString, AgentInfo> oldInfos = mAgents;
+  QHash<QString, AgentType> oldInfos = mAgents;
   readPluginInfos();
 
-  foreach ( const AgentInfo &oldInfo, oldInfos ) {
+  foreach ( const AgentType &oldInfo, oldInfos ) {
     if ( !mAgents.contains( oldInfo.identifier ) )
       emit agentTypeRemoved( oldInfo.identifier );
   }
 
-  foreach ( const AgentInfo &newInfo, mAgents ) {
+  foreach ( const AgentType &newInfo, mAgents ) {
     if ( !oldInfos.contains( newInfo.identifier ) ) {
       emit agentTypeAdded( newInfo.identifier );
       ensureAutoStart( newInfo );
@@ -347,7 +341,7 @@ void AgentManager::readPluginInfos( const QDir& directory )
   for ( int i = 0; i < files.count(); ++i ) {
     const QString fileName = directory.absoluteFilePath( files[ i ] );
 
-    AgentInfo agentInfo;
+    AgentType agentInfo;
     if ( agentInfo.load( fileName, this ) ) {
       if ( mAgents.contains( agentInfo.identifier ) ) {
         mTracer->error( QLatin1String( "AgentManager::updatePluginInfos" ),
@@ -403,10 +397,9 @@ void AgentManager::load()
       continue;
     }
 
-    AgentInstanceInfo instance;
-    instance.identifier = instanceIdentifier;
-    instance.agentType = agentType;
-    if ( instance.start( mAgents.value( agentType ), this ) )
+    AgentInstance::Ptr instance( new AgentInstance( this ) );
+    instance->setIdentifier( instanceIdentifier );
+    if ( instance->start( mAgents.value( agentType ) ) )
       mAgentInstances.insert( instanceIdentifier, instance );
     file.endGroup();
   }
@@ -419,14 +412,14 @@ void AgentManager::save()
   QSettings file( configPath( true ), QSettings::IniFormat );
 
   file.clear();
-  foreach ( const AgentInfo &info, mAgents )
+  foreach ( const AgentType &info, mAgents )
     info.save( &file );
 
   file.beginGroup( "Instances" );
 
-  foreach ( const AgentInstanceInfo &info, mAgentInstances ) {
-    file.beginGroup( info.identifier );
-    file.setValue( "AgentType", info.agentType );
+  foreach ( const AgentInstance::Ptr &inst, mAgentInstances ) {
+    file.beginGroup( inst->identifier() );
+    file.setValue( "AgentType", inst->agentType() );
     file.endGroup();
   }
 
@@ -450,47 +443,10 @@ void AgentManager::serviceOwnerChanged( const QString &name, const QString&, con
     if ( !mAgentInstances.contains( identifier ) )
       return;
 
-    const bool restarting = mAgentInstances[ identifier ].agentControlInterface != 0;
-    delete mAgentInstances[ identifier ].agentControlInterface;
-    delete mAgentInstances[ identifier ].agentStatusInterface;
-    mAgentInstances[ identifier ].agentControlInterface = 0;
-    mAgentInstances[ identifier ].agentStatusInterface = 0;
-
-    org::freedesktop::Akonadi::Agent::Control *agentControlIface =
-      new org::freedesktop::Akonadi::Agent::Control( "org.freedesktop.Akonadi.Agent." + identifier,
-                                                     "/", QDBusConnection::sessionBus(), this );
-    if ( !agentControlIface || !agentControlIface->isValid() ) {
-      mTracer->error( QLatin1String( "AgentManager::resourceRegistered" ),
-                      QString( "Cannot connect to agent instance with identifier '%1', error message: '%2'" )
-                          .arg( identifier, agentControlIface ? agentControlIface->lastError().message() : "" ) );
+    AgentInstance::Ptr instance = mAgentInstances.value( identifier );
+    const bool restarting = instance->hasAgentInterface();
+    if ( !instance->obtainAgentInterface() )
       return;
-    }
-
-    agentControlIface->setObjectName( identifier );
-    mAgentInstances[ identifier ].agentControlInterface = agentControlIface;
-
-
-    org::freedesktop::Akonadi::Agent::Status *agentStatusIface =
-      new org::freedesktop::Akonadi::Agent::Status( "org.freedesktop.Akonadi.Agent." + identifier,
-                                                    "/", QDBusConnection::sessionBus(), this );
-    if ( !agentStatusIface || !agentStatusIface->isValid() ) {
-      mTracer->error( QLatin1String( "AgentManager::resourceRegistered" ),
-                      QString( "Cannot connect to agent instance with identifier '%1', error message: '%2'" )
-                          .arg( identifier, agentStatusIface ? agentStatusIface->lastError().message() : "" ) );
-      return;
-    }
-
-    agentStatusIface->setObjectName( identifier );
-    mAgentInstances[ identifier ].agentStatusInterface = agentStatusIface;
-
-    connect( agentStatusIface, SIGNAL( status( int, const QString& ) ),
-            this, SLOT( status( int, const QString& ) ) );
-    connect( agentStatusIface, SIGNAL( percent( int ) ),
-            this, SLOT( percent( int ) ) );
-    connect( agentStatusIface, SIGNAL( warning( const QString& ) ),
-            this, SLOT( warning( const QString& ) ) );
-    connect( agentStatusIface, SIGNAL( error( const QString& ) ),
-            this, SLOT( error( const QString& ) ) );
 
     if ( !restarting )
       emit agentInstanceAdded( identifier );
@@ -504,128 +460,17 @@ void AgentManager::serviceOwnerChanged( const QString &name, const QString&, con
     if ( !mAgentInstances.contains( identifier ) )
       return;
 
-    delete mAgentInstances[ identifier ].resourceInterface;
-    mAgentInstances[ identifier ].resourceInterface = 0;
-
-    org::freedesktop::Akonadi::Resource *resInterface =
-      new org::freedesktop::Akonadi::Resource( "org.freedesktop.Akonadi.Resource." + identifier,
-                                               "/", QDBusConnection::sessionBus(), this );
-
-    if ( !resInterface || !resInterface->isValid() ) {
-      mTracer->error( QLatin1String( "AgentManager::resourceRegistered" ),
-                      QString( "Cannot connect to agent instance with identifier '%1', error message: '%2'" )
-                          .arg( identifier, resInterface ? resInterface->lastError().message() : "" ) );
-      return;
-    }
-
-    resInterface->setObjectName( identifier );
-    connect( resInterface, SIGNAL( nameChanged( const QString& ) ),
-            this, SLOT( resourceNameChanged( const QString& ) ) );
-    mAgentInstances[ identifier ].resourceInterface = resInterface;
+    mAgentInstances.value( identifier )->obtainResourceInterface();
   }
 }
 
-void AgentManager::status( int status, const QString &message )
-{
-  org::freedesktop::Akonadi::Agent::Status *interface = dynamic_cast<org::freedesktop::Akonadi::Agent::Status*>( sender() );
-  if ( !interface ) {
-    mTracer->error( QLatin1String( "AgentManager::status" ),
-                    QLatin1String( "Got signal from unknown sender" ) );
-    return;
-  }
-
-  const QString identifier = interface->objectName();
-  if ( identifier.isEmpty() ) {
-    mTracer->error( QLatin1String( "AgentManager::status" ),
-                    QLatin1String( "Sender of status signal has no identifier" ) );
-    return;
-  }
-
-  emit agentInstanceStatusChanged( identifier, status, message );
-}
-
-void AgentManager::percent( int progress )
-{
-  org::freedesktop::Akonadi::Agent::Status *interface = dynamic_cast<org::freedesktop::Akonadi::Agent::Status*>( sender() );
-  if ( !interface ) {
-    mTracer->error( QLatin1String( "AgentManager::percent" ),
-                    QLatin1String( "Got signal from unknown sender" ) );
-    return;
-  }
-
-  const QString identifier = interface->objectName();
-  if ( identifier.isEmpty() ) {
-    mTracer->error( QLatin1String( "AgentManager::percent" ),
-                    QLatin1String( "Sender of percent signal has no identifier" ) );
-    return;
-  }
-
-  emit agentInstanceProgressChanged( identifier, progress, QString() );
-}
-
-void AgentManager::warning( const QString &message )
-{
-  org::freedesktop::Akonadi::Agent::Status *interface = dynamic_cast<org::freedesktop::Akonadi::Agent::Status*>( sender() );
-  if ( !interface ) {
-    mTracer->error( QLatin1String( "AgentManager::warning" ),
-                    QLatin1String( "Got signal from unknown sender" ) );
-    return;
-  }
-
-  const QString identifier = interface->objectName();
-  if ( identifier.isEmpty() ) {
-    mTracer->error( QLatin1String( "AgentManager::warning" ),
-                    QLatin1String( "Sender of warning signal has no identifier" ) );
-    return;
-  }
-
-  emit agentInstanceWarning( identifier, message );
-}
-
-void AgentManager::error( const QString &message )
-{
-  org::freedesktop::Akonadi::Agent::Status *interface = dynamic_cast<org::freedesktop::Akonadi::Agent::Status*>( sender() );
-  if ( !interface ) {
-    mTracer->error( QLatin1String( "AgentManager::error" ),
-                    QLatin1String( "Got signal from unknown sender" ) );
-    return;
-  }
-
-  const QString identifier = interface->objectName();
-  if ( identifier.isEmpty() ) {
-    mTracer->error( QLatin1String( "AgentManager::error" ),
-                    QLatin1String( "Sender of error signal has no identifier" ) );
-    return;
-  }
-
-  emit agentInstanceError( identifier, message );
-}
-
-void AgentManager::resourceNameChanged( const QString &data )
-{
-  org::freedesktop::Akonadi::Resource *resource = dynamic_cast<org::freedesktop::Akonadi::Resource*>( sender() );
-  if ( !resource ) {
-    mTracer->error( QLatin1String( "AgentManager::resourceNameChanged" ),
-                    QLatin1String( "Got signal from unknown sender" ) );
-    return;
-  }
-
-  const QString identifier = resource->objectName();
-  if ( identifier.isEmpty() ) {
-    mTracer->error( QLatin1String( "AgentManager::resourceNameChanged" ),
-                    QLatin1String( "Sender of resourceNameChanged signal has no identifier" ) );
-    return;
-  }
-
-  emit agentInstanceNameChanged( identifier, data );
-}
-
 bool AgentManager::checkInstance(const QString & identifier) const
 {
   if ( !mAgentInstances.contains( identifier ) ) {
     qWarning() << "Agent instance with identifier " << identifier << " does not exist";
     return false;
   }
+
   return true;
 }
 
@@ -633,9 +478,9 @@ bool AgentManager::checkResourceInterface( const QString &identifier, const QStr
 {
   if ( !checkInstance( identifier ) )
     return false;
-  if ( !mAgents[ mAgentInstances[ identifier ].agentType ].capabilities.contains( "Resource" ) )
+  if ( !mAgents[ mAgentInstances[ identifier ]->agentType() ].capabilities.contains( "Resource" ) )
     return false;
-  if ( !mAgentInstances[ identifier ].resourceInterface ) {
+  if ( !mAgentInstances[ identifier ]->hasResourceInterface() ) {
     qWarning() << QLatin1String( "AgentManager::" ) + method << " Agent instance "
         << identifier << " has no resource interface!";
     return false;
@@ -657,28 +502,23 @@ bool AgentManager::checkAgentInterfaces(const QString & identifier, const QStrin
 {
   if ( !checkInstance( identifier ) )
     return false;
-  if ( !mAgentInstances.value( identifier ).agentControlInterface ) {
-    qWarning() << "Agent instance (" << method << ") " << identifier << " has no agent control interface.";
-    return false;
-  }
-  if ( !mAgentInstances.value( identifier ).agentStatusInterface ) {
-    qWarning() << "Agent instance (" << method << ") " << identifier << " has no agent status interface.";
+  if ( !mAgentInstances.value( identifier )->hasAgentInterface() ) {
+    qWarning() << "Agent instance (" << method << ") " << identifier << " has no agent interface.";
     return false;
   }
   return true;
 }
 
-void AgentManager::ensureAutoStart(const AgentInfo & info)
+void AgentManager::ensureAutoStart(const AgentType & info)
 {
-  if ( !info.capabilities.contains( AgentInfo::CapabilityAutostart ) )
+  if ( !info.capabilities.contains( AgentType::CapabilityAutostart ) )
     return; // no an autostart agent
   if ( mAgentInstances.contains( info.identifier ) )
     return; // already running
-  AgentInstanceInfo instance;
-  instance.identifier = info.identifier;
-  instance.agentType = info.identifier;
-  if ( instance.start( info, this ) ) {
-    mAgentInstances.insert( instance.identifier, instance );
+  AgentInstance::Ptr instance( new AgentInstance( this ) );
+  instance->setIdentifier( info.identifier );
+  if ( instance->start( info ) ) {
+    mAgentInstances.insert( instance->identifier(), instance );
     save();
   }
 }
