@@ -25,7 +25,8 @@
 #include <QtCore/QMap>
 #include <QtCore/QStringList>
 
-#include "agentinfo.h"
+#include "agenttype.h"
+#include "agentinstance.h"
 #include "tracerinterface.h"
 
 class QDir;
@@ -42,7 +43,7 @@ namespace Akonadi {
 class AgentManager : public QObject
 {
   Q_OBJECT
-  Q_CLASSINFO( "D-Bus Interface", "org.kde.Akonadi.AgentManager" )
+  Q_CLASSINFO( "D-Bus Interface", "org.freedesktop.Akonadi.AgentManager" )
 
   public:
     /**
@@ -68,7 +69,7 @@ class AgentManager : public QObject
      */
     static QString configPath( bool writeable );
 
-    org::kde::Akonadi::Tracer* tracer() const { return mTracer; }
+    org::freedesktop::Akonadi::Tracer* tracer() const { return mTracer; }
 
   public Q_SLOTS:
     /**
@@ -275,14 +276,14 @@ class AgentManager : public QObject
      */
     void agentInstanceNameChanged( const QString &agentIdentifier, const QString &name );
 
+    /**
+     * Emitted when the online state of an agent changed.
+     */
+    void agentInstanceOnlineChanged( const QString &agentIdentifier, bool state );
+
   private Q_SLOTS:
     void updatePluginInfos();
     void serviceOwnerChanged( const QString&, const QString&, const QString& );
-    void status( int status, const QString &message );
-    void percent( int progress );
-    void warning( const QString &message );
-    void error( const QString &message );
-    void resourceNameChanged( const QString& );
 
   private:
     /**
@@ -317,7 +318,7 @@ class AgentManager : public QObject
     bool checkInstance( const QString &identifier ) const;
     bool checkResourceInterface( const QString &identifier, const QString &method ) const;
     bool checkAgentExists( const QString &identifier ) const;
-    void ensureAutoStart( const AgentInfo &info );
+    void ensureAutoStart( const AgentType &info );
     void continueStartup();
 
   private:
@@ -327,18 +328,20 @@ class AgentManager : public QObject
      *
      * Key is the agent type (e.g. 'file' or 'imap').
      */
-    QHash<QString, AgentInfo> mAgents;
+    QHash<QString, AgentType> mAgents;
 
     /**
      * The map which stores the active instances.
      *
      * Key is the instance identifier.
      */
-    QHash<QString, AgentInstanceInfo> mAgentInstances;
+    QHash<QString, AgentInstance::Ptr> mAgentInstances;
 
-    org::kde::Akonadi::Tracer *mTracer;
+    org::freedesktop::Akonadi::Tracer *mTracer;
 
     Akonadi::ProcessControl *mStorageController;
+
+    friend class AgentInstance;
 };
 
 #endif
