@@ -24,9 +24,10 @@
 #include <QtDBus/QDBusError>
 
 #include "akonadi.h"
-#include <cstdlib>
+#include "akdebug.h"
+#include "akcrash.h"
 
-#include "../control/kcrash.h"
+#include <cstdlib>
 
 void shutdownHandler( int )
 {
@@ -39,8 +40,8 @@ void shutdownHandler( int )
 
 int main( int argc, char ** argv )
 {
-    KCrash::init();
-    KCrash::setShutdownMethod( shutdownHandler );
+    akInit( QString::fromLatin1( "akonadiserver" ) );
+    AkonadiCrash::setShutdownMethod( shutdownHandler );
 
     QCoreApplication app( argc, argv );
 
@@ -51,9 +52,9 @@ int main( int argc, char ** argv )
       return 1;
     }
 
-    app.exec();
+    const int result = app.exec();
 
     Akonadi::AkonadiServer::instance()->quit();
 
-    return 0;
+    return result;
 }
